require 'rails_helper'

module Logistics
  module Core
    RSpec.describe Agency, type: :model do
      it 'has a valid factory' do
        expect(create(:agency)).to be_valid
      end

      it 'is invalid without a code' do
        expect(build(:agency, code: nil)).not_to be_valid
      end

      it 'is invalid without a name' do
        expect(build(:agency, name: nil)).not_to be_valid
      end

      it 'is invalid with a duplicate code and agency category id' do
        agency = create(:agency)
        expect(build(:agency, code: agency.code, name: agency.name, agency_category: agency.agency_category)).not_to be_valid
      end

      it 'is invalid with telephone number length greater than 13' do
        expect(build(:agency, telephone: '+251911191929222')).not_to be_valid
      end
    end
  end
end