require 'rails_helper'

module Logistics
  module Core
    RSpec.describe EslTransportTariffRate, type: :model do
      it 'has a valid factory' do
        expect(create(:esl_transport_tariff_rate)).to be_valid
      end

      it 'is invalid with no esl transport tariff' do
        expect(build(:esl_transport_tariff_rate, :esl_transport_tariff => nil)).not_to be_valid
      end

      it 'is invalid with no qt from' do
        expect(build(:esl_transport_tariff_rate, :qt_from => nil)).not_to be_valid
      end

      it 'is invalid with no qt to' do
        expect(build(:esl_transport_tariff_rate, :qt_to => nil)).not_to be_valid
      end

      it 'is invalid with no rate' do
        expect(build(:esl_transport_tariff_rate, :rate => nil)).not_to be_valid
      end

      it 'is invalid with duplicate transport tariff, qt from and qt to' do
        ttr = create(:esl_transport_tariff_rate)
        expect(build(:esl_transport_tariff_rate, :esl_transport_tariff => ttr.esl_transport_tariff,
                     :qt_from => ttr.qt_from, :qt_to => ttr.qt_to)).not_to be_valid
      end
    end
  end
end